@@ -9,12 +9,12 @@
 
 #pragma once
 
+#include <vector>
 #include <memory>
 #include <cstdint>
 #include <iostream>
 
 #include "bb/Assert.h"
-#include "bb/SimdSupport.h"
 
 
 namespace bb {
@@ -45,6 +45,9 @@ namespace bb {
 #define BB_TYPE_UINT64          (0x0300 + 64)
 
 
+#define BB_BINARY_HI            (+1)
+#define BB_BINARY_LO            (-1)
+
 
 
 // Index型 (配列等の添え字用、符号付き size_t としての扱い)
@@ -80,9 +83,9 @@ inline index_t CalcShapeSize(indices_t const & shape)
 
 inline bool CalcNextIndices(indices_t& indices, indices_t const & shape)
 {
-    index_t size = indices.size();
+    std::intptr_t size = (std::intptr_t)indices.size();
     BB_DEBUG_ASSERT(shape.size() == size);
-    for ( index_t i = size-1; i >= 0; --i) {
+    for ( std::intptr_t i = size-1; i >= 0; --i) {
         if ( ++indices[i] < shape[i] ) {
             return true;
         }
@@ -98,14 +101,14 @@ inline indices_t RegurerlizeIndices(std::vector<T> indices, indices_t const &sha
     BB_ASSERT(indices.size() == shape.size());
 
     // 負数の折り返し
-    for ( index_t i = (index_t)indices.size()-1; i >= 0;  --i ) {
+    for ( std::intptr_t i = (std::intptr_t)indices.size()-1; i >= 0;  --i ) {
         while ( indices[i] < 0 ) {
             indices[i] += (T)shape[i];
         }
     }
 
     indices_t   reg_indices(indices.size());
-    for ( index_t i = (index_t)indices.size()-1; i >= 0;  --i ) {
+    for ( std::intptr_t i = (std::intptr_t)indices.size()-1; i >= 0;  --i ) {
         reg_indices[i] = (index_t)indices[i] % shape[i];
     }
 
@@ -118,7 +121,7 @@ inline indices_t ConvertIndexToIndices(index_t index, indices_t const & shape)
     indices_t indices(shape.size());
     BB_DEBUG_ASSERT(index >= 0 && index < CalcShapeSize(shape));
 
-    for ( index_t i = (index_t)shape.size()-1; i >= 0;  --i ) {
+    for ( std::intptr_t i = (std::intptr_t)shape.size()-1; i >= 0;  --i ) {
         indices[i] = index % shape[i];
         index /= shape[i];
     }
@@ -132,7 +135,7 @@ inline index_t ConvertIndicesToIndex(indices_t const & indices, indices_t const
 
     index_t stride = 1;
     index_t index = 0;
-    for ( index_t i = (index_t)shape.size()-1; i >= 0;  --i ) {
+    for ( std::intptr_t i = (std::intptr_t)shape.size()-1; i >= 0;  --i ) {
         BB_DEBUG_ASSERT(indices[i] >= 0 && indices[i] < shape[i]);
         index += stride * indices[i];
         stride *= shape[i];
@@ -174,7 +177,7 @@ inline index_t CalcIndexWithStride(indices_t const & indices, indices_t const &
     BB_DEBUG_ASSERT(indices.size() == stride.size());
 
     index_t index = 0;
-    for ( index_t i = (index_t)stride.size()-1; i >= 0;  --i ) {
+    for ( std::intptr_t i = (std::intptr_t)stride.size()-1; i >= 0;  --i ) {
         BB_DEBUG_ASSERT(indices[i] >= 0);
         index += stride[i] * indices[i];
     }
@@ -186,7 +189,7 @@ inline index_t CalcIndexWithStride(indices_t const & indices, indices_t const &
     BB_DEBUG_ASSERT(indices.size() == shape.size());
 
     index_t index = 0;
-    for ( index_t i = 0; i < (index_t)shape.size(); ++i ) {
+    for ( std::intptr_t i = 0; i < (std::intptr_t)shape.size(); ++i ) {
         BB_DEBUG_ASSERT(indices[i] >= 0 && indices[i] < shape[i]);
         index += stride[i] * indices[i];
     }
@@ -213,7 +216,7 @@ inline void SaveIndices(std::ostream &os, indices_t const &indices)
 {
     auto size = (index_t)indices.size();
     SaveIndex(os, (index_t)indices.size());
-    for (index_t i = 0; i < size; ++i) {
+    for (std::intptr_t i = 0; i < (std::intptr_t)size; ++i) {
         SaveIndex(os, indices[i]);
     }
 }
@@ -292,8 +295,22 @@ public:
     bool operator<(Bit const &bit) { return ((int)m_value < (int)bit.m_value); }
     bool operator<=(Bit const &bit) { return ((int)m_value <= (int)bit.m_value); }
 
+#if 0
     template<typename Tp>
-    operator Tp() { return m_value ? (Tp)1.0 : (Tp)0.0; }
+    operator Tp() { return m_value ? (Tp)BB_BINARY_HI : (Tp)BB_BINARY_LO; }
+#else
+    explicit operator double()        const { return m_value ? (double      )BB_BINARY_HI : (double      )BB_BINARY_LO; }
+    explicit operator float()         const { return m_value ? (float       )BB_BINARY_HI : (float       )BB_BINARY_LO; }
+    explicit operator std::int64_t()  const { return m_value ? (std::int64_t)BB_BINARY_HI : (std::int64_t)BB_BINARY_LO; }
+    explicit operator std::int32_t()  const { return m_value ? (std::int32_t)BB_BINARY_HI : (std::int32_t)BB_BINARY_LO; }
+    explicit operator std::int16_t()  const { return m_value ? (std::int16_t)BB_BINARY_HI : (std::int16_t)BB_BINARY_LO; }
+    explicit operator std::int8_t()   const { return m_value ? (std::int8_t )BB_BINARY_HI : (std::int8_t )BB_BINARY_LO; }
+    explicit operator std::uint64_t() const { return m_value ? 1 : 0; }
+    explicit operator std::uint32_t() const { return m_value ? 1 : 0; }
+    explicit operator std::uint16_t() const { return m_value ? 1 : 0; }
+    explicit operator std::uint8_t()  const { return m_value ? 1 : 0; }
+#endif
+
     operator bool() const { return m_value; }
 };
 
@@ -679,7 +696,7 @@ inline void LoadValue(std::istream &is, std::vector<T>  &vec)
 {
     std::uint64_t size;
     is.read((char *)&size, sizeof(size));
-    vec.resize(size);
+    vec.resize((std::size_t)size);
     if ( size > 0 ) {
         is.read((char *)&vec[0], size*sizeof(T));
     }
@@ -701,7 +718,7 @@ inline void LoadValue(std::istream &is, std::string &str)
 {
     std::uint64_t size;
     is.read((char *)&size, sizeof(size));
-    str.resize(size);
+    str.resize((std::size_t)size);
     if ( size > 0 ) {
         is.read((char *)&str[0], size*sizeof(str[0]));
     }
