@@ -198,33 +198,33 @@ void TrainDataNormalize(TrainData<T> &td)
     auto x_size = CalcShapeSize(td.x_shape);
     auto t_size = CalcShapeSize(td.t_shape);
 
-    std::vector<T>  x_max(x_size, (T)1.0);
-    std::vector<T>  x_min(x_size, (T)0.0);
-    std::vector<T>  t_max(t_size, (T)1.0);
-    std::vector<T>  t_min(t_size, (T)0.0);
+    std::vector<T>  x_max((std::size_t)x_size, (T)1.0);
+    std::vector<T>  x_min((std::size_t)x_size, (T)0.0);
+    std::vector<T>  t_max((std::size_t)t_size, (T)1.0);
+    std::vector<T>  t_min((std::size_t)t_size, (T)0.0);
 
     // 集計
     for (auto& x : td.x_train) {
-        for (index_t i = 0; i < x_size; ++i) {
+        for (std::size_t i = 0; i < (std::size_t)x_size; ++i) {
             x_max[i] = std::max(x_max[i], x[i]);
             x_min[i] = std::min(x_min[i], x[i]);
         }
     }
     for (auto& x : td.x_test) {
-        for (index_t i = 0; i < x_size; ++i) {
+        for (std::size_t i = 0; i < (std::size_t)x_size; ++i) {
             x_max[i] = std::max(x_max[i], x[i]);
             x_min[i] = std::min(x_min[i], x[i]);
         }
     }
 
     for (auto& t : td.t_train) {
-        for (index_t i = 0; i < t_size; ++i) {
+        for (std::size_t i = 0; i < (std::size_t)t_size; ++i) {
             t_max[i] = std::max(t_max[i], t[i]);
             t_min[i] = std::min(t_min[i], t[i]);
         }
     }
     for (auto& t : td.t_test) {
-        for (index_t i = 0; i < t_size; ++i) {
+        for (std::size_t i = 0; i < (std::size_t)t_size; ++i) {
             t_max[i] = std::max(t_max[i], t[i]);
             t_min[i] = std::min(t_min[i], t[i]);
         }
@@ -232,23 +232,23 @@ void TrainDataNormalize(TrainData<T> &td)
 
     // 正規化
     for (auto& x : td.x_train) {
-        for (index_t i = 0; i < x_size; ++i) {
+        for (std::size_t i = 0; i < (std::size_t)x_size; ++i) {
             x[i] = (x[i] - x_min[i]) / (x_max[i] - x_min[i]);
         }
     }
     for (auto& x : td.x_test) {
-        for (index_t i = 0; i < x_size; ++i) {
+        for (std::size_t i = 0; i < (std::size_t)x_size; ++i) {
             x[i] = (x[i] - x_min[i]) / (x_max[i] - x_min[i]);
         }
     }
 
     for (auto& t : td.t_train) {
-        for (index_t i = 0; i < t_size; ++i) {
+        for (std::size_t i = 0; i < (std::size_t)t_size; ++i) {
             t[i] = (t[i] - t_min[i]) / (t_max[i] - t_min[i]);
         }
     }
     for (auto& t : td.t_test) {
-        for (index_t i = 0; i < t_size; ++i) {
+        for (std::size_t i = 0; i < (std::size_t)t_size; ++i) {
             t[i] = (t[i] - t_min[i]) / (t_max[i] - t_min[i]);
         }
     }
@@ -336,7 +336,7 @@ void DumpMemory(std::string filename, T const *addr, size_t size)
 template<typename T> 
 void WriteTestDataBinTextFile(std::ostream& ofs, std::vector< std::vector<T> > x, std::vector< std::vector<T> > y)
 {
-    for (index_t i = 0; i < (index_t)x.size(); ++i) {
+    for (std::intptr_t i = 0; i < (std::intptr_t)x.size(); ++i) {
         auto yi = argmax<>(y[i]);
 
         for (int j = 7; j >= 0; --j) {
@@ -344,7 +344,7 @@ void WriteTestDataBinTextFile(std::ostream& ofs, std::vector< std::vector<T> > x
         }
         ofs << "_";
 
-        for (index_t j = (index_t)x[i].size()-1; j >= 0; --j) {
+        for (std::intptr_t j = (std::intptr_t)x[i].size()-1; j >= 0; --j) {
             if (x[i][j] > (T)0.5) {
                 ofs << "1";
             }
