@@ -78,9 +78,31 @@ class Tensor(bb.Object):
     def set_numpy(self, ndarray: np.ndarray):
         dtype = self.get_core().get_type()
         assert(bb.dtype_numpy_to_bb(ndarray.dtype) == dtype)
-        assert(ndarray.shspe == self.get_shape)
-        # 未実装
+        assert(ndarray.shape == tuple(self.get_shape()))
 
+        if dtype == bb.DType.FP32:
+            return self.get_core().set_numpy_fp32(ndarray)
+        elif dtype == bb.DType.FP64:
+            return self.get_core().set_numpy_fp64(ndarray)
+        elif dtype == bb.DType.INT8:
+            return self.get_core().set_numpy_int8(ndarray)
+        elif dtype == bb.DType.INT16:
+            return self.get_core().set_numpy_int16(ndarray)
+        elif dtype == bb.DType.INT32:
+            return self.get_core().set_numpy_int32(ndarray)
+        elif dtype == bb.DType.INT64:
+            return self.get_core().set_numpy_int64(ndarray)
+        elif dtype == bb.DType.UINT8:
+            return self.get_core().set_numpy_uint8(ndarray)
+        elif dtype == bb.DType.UINT16:
+            return self.get_core().set_numpy_uint16(ndarray)
+        elif dtype == bb.DType.UINT32:
+            return self.get_core().set_numpy_uint32(ndarray)
+        elif dtype == bb.DType.UINT64:
+            return self.get_core().set_numpy_uint64(ndarray)
+        else:
+            assert(0)
+    
     @staticmethod
     def from_numpy(ndarray: np.ndarray, host_only=False):
         """NumPy から生成
@@ -118,7 +140,42 @@ class Tensor(bb.Object):
         
         return Tensor(core_tensor=core_tensor)
 
+    def fill_zero(self):
+        self.get_core().fill_zero()
+    
+    def fill(self, x):
+        self.get_core().fill(x)
 
+    def isnan(self):
+        return self.get_core().isnan()
+
+    def min(self):
+        return self.get_core().min()
+
+    def max(self):
+        return self.get_core().max()
+
+    def clamp_inplace(self, a, b):
+        self.get_core().clamp_inplace(a, b)
+    
+    def sqrt_inplace(self):
+        self.get_core().sqrt_inplace()
+
+    def exp_inplace(self):
+        self.get_core().exp_inplace()
+    
+    def sum(self):
+        return self.get_core().sum()
+
+    def mean(self):
+        return self.get_core().mean()
+    
+    def var(self):
+        return self.get_core().var()
+    
+    def std(self):
+        return self.get_core().std()
+    
     def __add__(self, x):
         if type(x) == Tensor:
             core_tensor = self.get_core() + x.get_core()
